'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _babelTypes = require('babel-types');

var t = _interopRequireWildcard(_babelTypes);

var _util = require('../util');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var getArrayItems = function getArrayItems(_ref) {
  var elements = _ref.elements;
  return _lodash2.default.map(elements, function (_ref2) {
    var value = _ref2.value;
    return value;
  });
};

var getObjectKeys = function getObjectKeys(_ref3) {
  var properties = _ref3.properties;

  var objectProperties = _lodash2.default.filter(properties, _util.isObjectProperty);

  return _lodash2.default.map(objectProperties, function (_ref4) {
    var _ref4$key = _ref4.key;
    _ref4$key = _ref4$key === undefined ? {} : _ref4$key;
    var name = _ref4$key.name;
    return name;
  });
};

var getTypeAlias = function getTypeAlias(path, name) {
  var body = _lodash2.default.get(path, 'parent.body') || [];
  var type = body.filter(function (item) {
    return _lodash2.default.get(item, 'id.name') === name;
  });
  return _lodash2.default.get(type, '0');
};

var getParams = function getParams(path) {
  var params = _lodash2.default.get(path, 'node.right.typeParameters.params') || [];
  return params.map(function (item) {
    return _lodash2.default.get(item, 'id.name');
  });
};

var getTypesByProperties = function getTypesByProperties(properties) {
  return properties.map(function (item) {
    var type = _lodash2.default.get(item, 'key.type');
    if (type === 'StringLiteral') {
      return _lodash2.default.get(item, 'key.value');
    } else if (type === 'Identifier') {
      return _lodash2.default.get(item, 'key.name');
    }
    return;
  });
};

var getFlowTypeKeys = function getFlowTypeKeys(path) {
  if (_lodash2.default.get(path, 'node.right.id.name') === '$Diff') {
    var params = getParams(path) || [];
    var types = [];
    params.map(function (name) {
      var item = getTypeAlias(path, name);
      types.push.apply(types, _toConsumableArray(getTypesByProperties(_lodash2.default.get(item, 'right.properties') || [])));
    });
    return types;
  }

  if (_lodash2.default.get(path, 'node.id.name') !== 'Props') {
    return [];
  }

  var properties = _lodash2.default.get(path, 'node.right.properties') || [];

  return getTypesByProperties(properties);
};

var getTypeAliasIdentifier = function getTypeAliasIdentifier(path) {
  var body = _lodash2.default.get(path, 'parent.body');
  if (!body) {
    return Object.keys(_lodash2.default.get(path, 'state.entries'))[0];
  }
  return _lodash2.default.get(body.find(function (item) {
    return item.type === 'ClassDeclaration';
  }), 'id.name');
};

var propVisitor = {
  TypeAlias(path, state) {
    var identifier = getTypeAliasIdentifier(path);

    if (!state.hasEntry(identifier)) return;

    state.addProps(identifier, getFlowTypeKeys(path));
  },
  AssignmentExpression(path, state) {
    var identifier = (0, _util.getExpressionIdentifier)(path);
    var right = _lodash2.default.get(path, 'node.right');

    if (!state.hasEntry(identifier)) return;

    if ((0, _util.isValidExpression)(path, ['handledProps']) && (0, _util.isArrayValue)(right)) {
      state.addProps(identifier, getArrayItems(right));
      path.remove();
      return;
    }

    if ((0, _util.isValidExpression)(path, ['defaultProps', 'propTypes'])) {
      if ((0, _util.isObjectValue)(right)) {
        state.addProps(identifier, getObjectKeys(right));
      } else if (t.isIdentifier(right)) {
        var name = _lodash2.default.get(right, 'name');
        var properties = _lodash2.default.get(path, `scope.bindings.${name}.path.node.init.properties`) || [];
        var keys = properties.map(function (item) {
          return _lodash2.default.get(item, 'key.name');
        });
        state.addProps(identifier, keys);
      }
    }
  },
  ClassProperty(path, state) {
    var expression = (0, _util.getClassDeclaration)(path);
    var identifier = (0, _util.getEntryIdentifier)(expression);
    var value = _lodash2.default.get(path, 'node.value');

    if (!state.hasEntry(identifier) || !(0, _util.isStaticProperty)(path)) return;

    if ((0, _util.isValidProperty)(path, ['handledProps']) && (0, _util.isArrayValue)(value)) {
      state.addProps(identifier, getArrayItems(value));
      path.remove();

      return;
    }

    if ((0, _util.isValidProperty)(path, ['defaultProps', 'propTypes']) && (0, _util.isObjectValue)(value)) {
      state.addProps(identifier, getObjectKeys(value));
    }
  }
};

exports.default = propVisitor;